#include <stdlib.h>

#include "local.h"
#include "grafi.h"

status init_graph(graph *p_G, int vertex_cnt, graph_type type)
{
	graph G;
	int i,j;

	G = (graph) malloc(sizeof(graph_header));
	if ( G == NULL ) return ERROR;

	G->number_of_vertices = vertex_cnt;
	G->type = type;

	G->matrix = (edge **)malloc(vertex_cnt * sizeof(edge *));
	if (G->matrix == NULL) {
		free(G);
		return ERROR;
	}
	G->matrix[0] = (edge *) malloc(vertex_cnt*vertex_cnt*sizeof(edge));
	if (G->matrix[0] == NULL) {
		free(G->matrix);
		free(G);
		return ERROR;
	}

	for (i =0; i< vertex_cnt; i++)
		G->matrix[i] = G->matrix[0] + vertex_cnt * i;

	for (i =0; i < vertex_cnt; i++) {
		for (j=0; j < vertex_cnt; j++) {
			G->matrix[i][j].weight = UNUSED_WEIGHT;
			G->matrix[i][j].vertex_number = j;
		}
	} 

	*p_G = G;
	return OK;
}


void destroy_graph(graph *p_G)
{
	free((*p_G)->matrix[0]);
	free((*p_G)->matrix);
	*p_G = NULL;
}

status add_edge(graph G, vertex vertex1, vertex vertex2, int weight)
{	
	if (vertex1<0 || vertex1 >=G->number_of_vertices)
		return ERROR;
	if (vertex2<0 || vertex2 >=G->number_of_vertices)
		return ERROR;
	if (weight>=UNUSED_WEIGHT)
		return ERROR;

	G->matrix[vertex1][vertex2].weight = weight;
	if ( (G->type == undirected) & (vertex1 != vertex2))
		G->matrix[vertex2][vertex1].weight = weight;

	return OK;
}

status delete_edge(graph G, vertex vertex1, vertex vertex2)
{
	if (vertex1<0 || vertex1 >=G->number_of_vertices)
		return ERROR;
	if (vertex2<0 || vertex2 >=G->number_of_vertices)
		return ERROR;

	G->matrix[vertex1][vertex2].weight = UNUSED_WEIGHT;
	if (G->type == undirected)
		G->matrix[vertex2][vertex1].weight = UNUSED_WEIGHT;

	return OK;
}

bool is_adjacent(graph G, vertex vertex1, vertex vertex2)
{
	if (vertex1<0 || vertex1 >=G->number_of_vertices)
		return FALSE;
	if (vertex2<0 || vertex2 >=G->number_of_vertices)
		return FALSE;

	return(G->matrix[vertex1][vertex2].weight == UNUSED_WEIGHT)? FALSE : TRUE;

}

void graph_size(graph G, int *p_vertex_cnt, int *p_edge_cnt)
{
	int i,j, edges;

	*p_vertex_cnt = G->number_of_vertices;

	edges = 0;
	for (i = 0; i< G->number_of_vertices; i++)
		for (j = 0; j< G->number_of_vertices; j++)
			if (G->matrix[i][j].weight != UNUSED_WEIGHT)
				edges++;

	if (G->type == undirected)
		edges = edges /2;
	*p_edge_cnt = edges;
}

edge *edge_iterator(graph G, vertex vertex_number, edge	*p_last_return)
{
	vertex other_vertex;

	if (vertex_number<0 || vertex_number >=G->number_of_vertices)
		return NULL;

	if (p_last_return == NULL)
		other_vertex = 0;
	else
		other_vertex = VERTEX(p_last_return) + 1;

	for ( ; other_vertex < G->number_of_vertices; other_vertex++)
		if (G->matrix[vertex_number][other_vertex].weight != UNUSED_WEIGHT)
			return &G->matrix[vertex_number][other_vertex];
	return NULL;
}
