/* LABORATORIO DI ALGORITMI

	gruppo LAUREA 01

	Cascio Vittorio 	st950339
	Manca Maurizio		st970729
	Borello Andrea		st970828
*/



#include "local.h"
#include "heap.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

status makeheap(heap *h,int n)
/* Inizializza in *h uno heap vuoto che puo' contenere n elementi;
restituisce un valore status a seconda che l'operazione sia andata
a buon fine oppure no*/
{
if(	h->vettore=calloc((size_t)n,sizeof(dato)) ) // se diverso da NULL
	{
	h->primalibera=0;
	h->maxsize=n-1;
	return ok;
	}
else
	return error;
}


status buildheap(heap *h)
/* Applica la proprieta' di heap ad *h */
{
	int i;
	if ( test_vuoto(*h))
		return error;
	for(i = parent(h->primalibera-1 ); i>=0; i--)
		heapify(h,i);

	return ok;
}


bool test_vuoto(heap h)
/* Controlla se h e' vuoto*/
{
	return !h.primalibera;
}

status inserisci(heap *h, dato d)
/* Inserisce il dato d in *h al posto giusto: restituisce un valore status
a seconda che l'operazione sia andata a buon fine oppure no*/
{
 int i;
 int k;

 if (h->primalibera > h->maxsize)
	 return error;

	i=(h->primalibera)++;
	k=d->priorita;
	while (i >0 && k ORD_REL h->vettore[parent(i)]->priorita )
	{
		h->vettore[i]=h->vettore[parent(i)];
		i=parent(i);
	}
	h->vettore[i]=d;

	return ok;

}

status estrai_minimo(heap *h, dato *d)
/* Estrae il minimo dello heap che viene letto in *d modificando
opportunamente lo heap; restituisce un valore status a seconda che
l'operazione sia andata a buon fine oppure no*/
{
   if (h->primalibera > 0 )
       {
       *d=h->vettore[0];
       h->vettore[0]=h->vettore[h->primalibera-1];
       h->primalibera--;
       heapify(h,0);
       return ok;
	}
    return error;
}

status minimo(heap h, dato *d)
/* Legge il minimo dello heap che viene letto in *d senza modificare lo heap;
restituisce un valore status a seconda che l'operazione sia andata a buon
fine oppure no (es. leggere il minimo di uno heap vuoto)*/
{
	if (h.primalibera )
		return error;
	else
		*d=h.vettore[0];
	return ok;
}

bool aggiorna(heap *h, int i, int nuovapriorita)
/* Aggiorna la priorita dell'elemento di indice i del vettore dello
heap a nuovapriorita se questa e' strettamente inferiore di quella
dell'elemento. Restituisce un valore booleano per indicare se la sostituzione
e' avvenuta o no. Osservare che lo heap e' passato per referenza anche se
di fatto la struttura heap non viene modificata*/
{
	dato temp;
	if(nuovapriorita < h->vettore[i]->priorita  )
	   {
		temp = h->vettore[i];
		h->vettore[i]->priorita=nuovapriorita;

		while (i > 0 && h->vettore[parent(i)]->priorita > nuovapriorita )
	   {
		h->vettore[i]=h->vettore[parent(i)];
		i=parent(i);
		}					//	end while
		h->vettore[i]=temp;
		return TRUE;
	}						//	end if

return FALSE;
}


void heapify(heap *h, int i)
/* Mantiene le proprieta' dello heap */
{
	int l,r,largest,last;
	l=left(i);
	r=right(i);
	last=h->primalibera-1;
	if ( l<=last && (h->vettore[l]->priorita) ORD_REL (h->vettore[i]->priorita))
		largest = l;
	else
		largest = i;
	if ( r<=last && (h->vettore[r]->priorita) ORD_REL (h->vettore[largest]->priorita))
		largest = r;
	if (largest != i)
	{
		swap( h,largest,i);
		heapify(h,largest);
	}

}

/*	per efficienza le seguenti funzioni sono state implementate con Macro nel file heap.h
__inline int parent(int i)
// Restituisce l'indice del padre del nodo di indice i
{
	return (i-1)/2;
}

__inline int left(int i)
// Restituisce l'indice del figlio sinistro del nodo con indice i
{
	return 2*i+1;
}

__inline int right(int i)
// Restituisce l'indice del figlio destro del nodo con indice i
{
	return 2*(i+1);
}

*/

void swap(heap *h, int a,int i)
/* Scambia i puntatori a due nodi dell'heap. L'indice a e' quello del figlio
da riportare in alto poiche' ha priorita' maggiore del padre*/
{
dato temp;

temp=h->vettore[a];
h->vettore[a]=h->vettore[i];
h->vettore[i]=temp;

}

int find(heap h, dato d)
/* Cerca nel vettore che costituisce lo heap un elemento il cui campo nome
sia uguale a quello di dato. Se lo trova restituisce l'indice relativo del
vettore, altrimenti restituisce -1*/
{
int i;
for(i=0;i<h.primalibera;i++)
	if ( strcmp(h.vettore[i]->nome,d->nome ) == 0)
		return i;
return -1;
}
void distruggi(heap *h)
/* Distrugge lo heap *h*/
{
int i;
for (i=0;i<h->primalibera;i++)
	distruggi_dato(h->vettore[i]);
free(h->vettore);
}

void distruggi_dato(dato d)
/*	Rilascia la memoria allocata per dato
	Questa procedura e stata aggiunta per rendere la gestione dello Heap
	Indipendente dal tipo di dato	*/
{
	free(d->nome);
	free(d);
}


void stampa(heap h)
/* Stampa il contenuto dello heap h*/
{
	int i;
	printf(" Stampa\nIndex\tNome\t\tPriorita'\n\n");
	for (i=0;i < h.primalibera;i++ )
			printf("%d\t%s\t %d\n",i,h.vettore[i]->nome,h.vettore[i]->priorita);

}